'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapReplaceReducer = exports.wrapDispatch = exports.wrapGetState = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _utils = require('./utils');

var _enhanceReducer = require('./enhanceReducer');

var _enhanceReducer2 = _interopRequireDefault(_enhanceReducer);

var _AppStateWithEffects = require('./AppStateWithEffects');

var _AppStateWithEffects2 = _interopRequireDefault(_AppStateWithEffects);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wraps Store `getState` method. Instead of returning just plain old state,
 * it's assumed that the returned state is instance of `AppStateWithEffects`,
 * therefore it proxies the call to `getAppState`.
 *
 * @param {Object} Original Store
 * @returns {Function} Wrapped `getState`
 */
var wrapGetState = exports.wrapGetState = function wrapGetState(store) {
  return function () {
    var state = store.getState();

    if (state instanceof _AppStateWithEffects2.default) {
      return state.getAppState();
    } else {
      return state;
    }
  };
};

/**
 * Wraps Store `dispatch` method. The original `dispatch` is called first, then
 * it iterates over all the effects returned from the `getState` function.
 * Every effect is then executed and dispatch is passed as the first argument.
 *
 * @param {Object} Original Store
 * @returns {Function} Wrapped `dispatch`
 */
var wrapDispatch = exports.wrapDispatch = function wrapDispatch(store) {
  return function () {
    // Let's just dispatch original action,
    // the original dispatch might actually be
    // enhanced by some middlewares.
    var result = store.dispatch.apply(store, arguments);

    var effects = store.getState().getEffects();

    (0, _utils.invariant)(effects.every(_utils.isFunction), 'It\'s allowed to yield only functions (side effect)');

    // Effects are executed after action is dispatched
    effects.forEach(function (effect) {
      return effect(wrapDispatch(store));
    });

    return result;
  };
};

/**
 * Wraps Store `replaceReducer` method. The implementation just calls
 * the original `replaceReducer` method but the provided next reducer
 * is enhanced.
 *
 * @param {Object} Original Store
 * @returns {Function} Wrapped `replaceReducer`
 */
var wrapReplaceReducer = exports.wrapReplaceReducer = function wrapReplaceReducer(store) {
  return function (nextReducer) {
    return store.replaceReducer((0, _enhanceReducer2.default)(nextReducer));
  };
};

/**
 * Creates enhanced store factory, which takes original `createStore` as argument.
 * The store's `dispatch` and `getState` methods are wrapped with custom implementation.
 *
 * wrappedDispatch calls the original dispatch and executes all the side effects.
 *
 * wrappedGetState unwraps original applicaiton state from `AppStateWithEffects`
 *
 * @param {Function} Original createStore implementation to be enhanced
 * @returns {Function} Store factory
 */

exports.default = function (createStore) {
  return function (rootReducer, initialState) {
    var store = createStore((0, _enhanceReducer2.default)(rootReducer), new _AppStateWithEffects2.default(initialState, []));

    return (0, _extends3.default)({}, store, {
      dispatch: wrapDispatch(store),
      getState: wrapGetState(store),
      liftGetState: function liftGetState() {
        return store.getState();
      },
      replaceReducer: wrapReplaceReducer(store)
    });
  };
};